import string, os.path
from qt import *

TRUE=1
FALSE=0

class CharsetSelector(QComboBox):

  def __init__(self, *args):
    apply(QComboBox.__init__,(self,)+args)
    self.charsets=[]
    self.connect(self, SIGNAL("activated(int)")
                     , self.sigActivated
                     )
    
    f=open(os.path.join(os.environ["KURADIR"], "charmap/unidata","Blocks.txt"))
    f.readline() # skip first line
    for line in f.readlines():
      try:
        self.charsets.append((string.atoi(line[0:4],16)
                             ,string.atoi(line[6:10],16)
                             )
                            )
        self.insertItem(line[12:])
      except: pass
      
  def sigActivated(self, index):
    begin, start=self.charsets[index]
    self.emit(PYSIGNAL("sigActivated"),(begin, start))

class CharsetCanvas(QCanvas):

  def __init__(self, parent, font, start, end, maxW, *args):
    #
    # NOTE: don't give the canvas a parent, otherwise Kura will
    # crash mysteriously with 
    #        Fatal Python error: PyThreadState_Get: no current thread 
    # when the canvas is recreated a few times.
    #
    # I think...
    #
    apply(QCanvas.__init__,(self, ) + args)
    self.start=start
    self.end=end
    self.font=font
    self.drawTable(maxW)

    
  def drawTable(self, maxW):
    self.maxW=maxW
    self.items=[]
    x=0
    y=0

    fontMetrics=QFontMetrics(self.font)
    cell_width=fontMetrics.maxWidth() + 3
    if self.maxW < 16 * cell_width: 
      self.maxW =  16 * cell_width
    cell_height=fontMetrics.lineSpacing()

    for wch in range(self.start, self.end + 1):
      item=QCanvasText(QString(QChar(wch)),self)
      item.setFont(self.font)
      
      item.setX(x)
      item.setY(y)
      item.show()
      
      self.items.append(item)
      
      x=x + cell_width
      if x > self.maxW:
        x=0
        y=y+cell_height
        
    self.resize(self.maxW + 20, y+cell_height)
    self.update()
    
  def setFont(self, font):
    self.font=font
    self.drawTable(self.maxW)
        
class CharsetBrowser(QCanvasView):

  def __init__(self, *args):
    apply(QCanvasView.__init__,(self,)+args)

  def setCursor(self, item):
    self.cursorItem=QCanvasRectangle(self.canvas()) 
    self.cursorItem.setX(item.boundingRect().x() -2)
    self.cursorItem.setY(item.boundingRect().y() -2)
    self.cursorItem.setSize(item.boundingRect().width() + 4, item.boundingRect().height() + 4)
    
    self.cursorItem.setZ(-1.0)
    self.cursorItem.setPen(QPen(QColor(Qt.gray), 2, Qt.DashLine))
    #self.cursorItem.setBrush(QBrush(Qt.yellow))
    self.cursorItem.show()
    self.canvas().update()
      
  def contentsMousePressEvent(self, ev):
    try:
      items=self.canvas().collisions(ev.pos())
      self.setCursor(items[0])      
      self.emit(PYSIGNAL("sigMousePressedOn"), (items[0].text(),))
    except IndexError:
      pass

  def setFont(self, font):
    self.font=font
    self.canvas().setFont(self.font)

class uniCharmap(QWidget):
  def __init__(self, parent, initialFont):
  
    QWidget.__init__(self, parent)
    self.parent=parent
    self.font=initialFont
    self.box=QVBoxLayout(self)
    self.comboCharset=CharsetSelector(FALSE, self)
    self.box.addWidget(self.comboCharset)
    self.charsetCanvas=CharsetCanvas(self, self.font, 0, 0, 0)
    self.charsetBrowser=CharsetBrowser(self.charsetCanvas, self)
    self.box.addWidget(self.charsetBrowser)


    self.connect(self.comboCharset, PYSIGNAL("sigActivated")
                                  , self.slotShowCharset
                                  )

    self.connect(self.charsetBrowser, PYSIGNAL("sigMousePressedOn")
                                , self.sigCharacterSelected
                                )
                                
    #self.comboCharset.sigActivated(self.comboCharset.currentItem())
    #self.slotShowCharset(0, 128)
                                        
  def setFont(self, font):
    self.font=font
    self.charsetBrowser.setFont(font)
                                
  def sigCharacterSelected(self, text):
    self.emit(PYSIGNAL("sigCharacterSelected"), (text,))
    
  def slotShowCharset(self, begin, end):
    self.setCursor(Qt.waitCursor)
    
    self.charTable=CharsetCanvas(self, self.font
                                     , begin, end
                                     , self.charsetBrowser.width() -20)

    self.charsetBrowser.setCanvas(self.charTable)
    self.setCursor(Qt.arrowCursor)    

__copyright__="""
/***************************************************************************
    copyright            : (C) 2000 by Boudewijn Rempt 
                           see copyright notice for license
    email                : boud@rempt.xs4all.nl
    Revision             : $Revision: 1.4 $
    Last edited          : $Date: 2001/01/30 19:35:31 $
    
    CVS Log:         
    $Log: charmap.py,v $
    Revision 1.4  2001/01/30 19:35:31  boud
    Fixed segfault when the charmap canvas was recreated a few times
    by _not_ giving the canvas a parent QObject.

    Revision 1.3  2001/01/08 20:54:56  boud
    Cleanup for version 1.0

 ***************************************************************************/
"""
