from qt import *
from pinyinmap import PinyinMap

TRUE=1
FALSE=0

class PinyinLine(QLineEdit):

  def keyPressEvent(self, ev):
    if ev.key()==Qt.Key_Return or ev.key()==Qt.Key_Enter:
      self.emit(PYSIGNAL("sigSyllableCompleted"), (unicode(self.text()),))
    QLineEdit.keyPressEvent(self, ev)

class CharacterCanvas(QCanvas):
  def __init__(self, parent, font, *args):
    apply(QCanvas.__init__,(self, parent,)+args)
    self.font=font
    self.characters=None
    self.maxW=0
    self.items=[]
    
  def setFont(self, font):
    self.font=font
    if len(self.items) > 0:
      self.drawTable(self.characters, self.maxW)
      
  def drawTable(self, characters, maxW):
    self.characters=characters
    self.maxW=maxW
    self.items=[]
    x=0
    y=0
    fontMetrics=QFontMetrics(self.font)
    cell_width=fontMetrics.maxWidth() + 5
    cell_height=fontMetrics.lineSpacing() + 5
    
    for ch in characters:
      item=QCanvasText(QString(ch), self)
      item.setFont(self.font)
      item.setX(x)
      item.setY(y)
      item.show()
      self.items.append(item)
      
      x=x+ cell_width
      if x > maxW:
        x=0
        y=y+cell_height
        
    self.resize(maxW, y+cell_height)
    self.update()

class CharacterPicker(QCanvasView):

  def __init__(self, font, *args):
    apply(QCanvasView.__init__,(self, ) + args)
    self.font=font

  def slotRefresh(self, pinyin):
    self.charCanvas=CharacterCanvas(self, self.font)
    self.setCanvas(self.charCanvas)
    try:
      self.canvas().drawTable(PinyinMap[pinyin.strip()], self.width() - 20)
    except KeyError:
      QMessageBox.warning(self, "Pinyin Input Method", "This is not valid Pinyin.")
    
  def setFont(self, font):
    self.font=font
    self.canvas().setFont(font)

  def setCursor(self, item):
    self.cursorItem=QCanvasRectangle(self.canvas()) 
    self.cursorItem.setX(item.boundingRect().x() -2)
    self.cursorItem.setY(item.boundingRect().y() -2)
    self.cursorItem.setSize(item.boundingRect().width() + 4, item.boundingRect().height() + 4)
    
    self.cursorItem.setZ(-1.0)
    self.cursorItem.setPen(QPen(QColor(Qt.gray), 2, Qt.DashLine))
    #self.cursorItem.setBrush(QBrush(Qt.yellow))
    self.cursorItem.show()
    self.canvas().update()

  def contentsMousePressEvent(self, ev):
    try:
      items=self.canvas().collisions(ev.pos())
      self.setCursor(items[0])      
      self.emit(PYSIGNAL("sigMousePressedOn"), (items[0].text(),))
    except IndexError:
      pass
    
class uniPinyin(QWidget): 
  
  def __init__(self, parent, initialFont):
    QWidget.__init__(self, parent)
    self.font=initialFont
    self.box=QVBoxLayout(self)
    self.pinyinLine=PinyinLine(self)
    self.box.addWidget(self.pinyinLine)
    self.characterCanvas=CharacterCanvas(parent, self.font)
    self.characterPicker=CharacterPicker(self.font, self.characterCanvas, self)
    self.box.addWidget(self.characterPicker)
    
    self.connect(self.pinyinLine, PYSIGNAL("sigSyllableCompleted")
                                , self.characterPicker.slotRefresh
                                )
    self.connect(self.characterPicker, PYSIGNAL("sigMousePressedOn")
                                     , self.sigCharacterSelected
                                     )
    self.connect(self.characterPicker, PYSIGNAL("sigMousePressedOn")
                                     , self.pinyinLine.clear
                                     )    
    self.pinyinLine.setFocus()
    
  def setFont(self, font):
    self.pinyinLine.setFont(font)
    self.characterPicker.setFont(font)
  
  def sigCharacterSelected(self, text):
    self.emit(PYSIGNAL("sigCharacterSelected"), (text,))
    self.pinyinLine.setFocus()
    

__copyright__="""
/***************************************************************************
    copyright            : (C) 2000 by Boudewijn Rempt 
                           see copyright notice for license
    email                : boud@rempt.xs4all.nl
    Revision             : $Revision: 1.3 $
    Last edited          : $Date: 2001/01/30 18:06:53 $
    
    CVS Log:         
    $Log: pinyin.py,v $
    Revision 1.3  2001/01/30 18:06:53  boud
    Made a package.

    Revision 1.2  2001/01/08 20:54:56  boud
    Cleanup for version 1.0

 ***************************************************************************/
"""
