import sys

from qt import *

from kuralib.config import Config
from kuralib.lng_lex import *
  
from kuragui.guitabdialog  import guiTabDialog
from kuragui.guilistview   import guiListView
from kuragui.guidetaillist import guiDetailList
from kuragui.guicombobox   import guiComboBox
from kuragui.constants     import *
from kuragui.guilineedit   import guiLineEdit

from tabelementtags        import tabElementTags
from wdglexeme             import tabLexeme
from dlglexeme             import dlgLexeme
from dlglexchooser         import dlgLexChooser
from resource import *

from dbobj.dbexceptions import dbError

class tabElementSplit(QWidget): 

  def __init__(self, parent, parentRecord):
    QWidget.__init__(self, parent)
    self.parentRecord=parentRecord
    
    wdgElementLayout = QGridLayout(self)
    wdgElementLayout.setSpacing(6)
    wdgElementLayout.setMargin(11)

    self.lblElement = QLabel(self,'lblElement')
    self.lblElement.setText("Separate the morphemes with a dot.")
    wdgElementLayout.addWidget(self.lblElement,0,0)
    
    self.txtElement = guiLineEdit(self)
    self.txtElement.setFont(Config.font)
    self.txtElement.setText(parentRecord.getFieldValue("text"))
    wdgElementLayout.addWidget(self.txtElement,0,1)
    
    self.lblSubType=QLabel(self)
    self.lblSubType.setText("Type")
    wdgElementLayout.addWidget(self.lblSubType, 1, 0)
    
    self.cmbSubType=guiComboBox(self)
    wdgElementLayout.addWidget(self.cmbSubType, 1, 1)
    self.cmbSubType.fillComboBox(self.parentRecord, "elementtypecode", INSERT)
    
    self.lblSeparate = QLabel(self,'lblSeparate')
    self.lblSeparate.setText(
      'Nota Bene: all current morpheme data for this element will be deleted.')
    self.lblSeparate.setAlignment(QLabel.WordBreak | 
                                  QLabel.AlignVCenter | 
                                  QLabel.AlignLeft)
    wdgElementLayout.addWidget(self.lblSeparate,2,0)
    
    self.bnSeparate = QPushButton(self,'bnSeparate')
    self.bnSeparate.setText('Create morphemes')
    self.connect(self.bnSeparate, SIGNAL("clicked()")
                                , self.slotSplitElement)
    wdgElementLayout.addWidget(self.bnSeparate,2,1)
    
    self.lsvChildElements = QListView(self,'lsvChildElements')
    self.lsvChildElements.setFont(Config.font)
    self.lsvChildElements.setSorting(-1, FALSE)
    self.lsvChildElements.setRootIsDecorated(TRUE)
    self.lsvChildElements.setShowSortIndicator(FALSE)
    self.lsvChildElements.setTreeStepSize(40)
    self.lsvChildElements.addColumn("Element")
    self.lsvChildElements.setColumnWidthMode(0, QListView.Maximum)
    
    self.connect(self.lsvChildElements, SIGNAL("returnPressed(QListViewItem *)")
                                      , self.slotItemSelected)
    self.connect(self.lsvChildElements, SIGNAL("doubleClicked(QListViewItem *)")
                                     , self.slotItemSelected)
    
    wdgElementLayout.addMultiCellWidget(self.lsvChildElements,3,3,0,1)
    self.refresh()

  def refresh(self):
    self.item_to_record={}
    self.record_to_item={}
    self.lsvChildElements.clear()
    self.addChildren(self.lsvChildElements, self.parentRecord.elementnr)
  
  def addChildren(self, parent, elementnr):
    previous=None
    for elmt in Config.app.getObjects("lng_element"
                                    , parent_elementnr=elementnr):
      if previous==None:
        item=QListViewItem(parent)
      else:
        item=QListViewItem(parent, previous)
      item.setText(0, elmt.getFieldValue("text"))
      previous=item
      self.item_to_record[item]=elmt
      self.record_to_item[elmt]=item
      self.addChildren(item, elmt.elementnr)
          
  def slotSplitElement(self):
      
    if self.lsvChildElements.childCount() > 0:
      if QMessageBox.warning(self, "Kura"
         , "This element already has sub-elements. Do you want to delete them?"
         , "Yes", "No"
         ) == 1:
        return
    try:        
      self.parentRecord.deleteChildren("lng_element")
    except dbError, dberr:
      QMessageBox.information(self
            , "Error while deleting"
            , dberr.errorMessage)

    els=unicode(self.txtElement.text()).split(".")
            
    if len(els) > 1:
      seqnr=0
      for element in els:
        record=Config.app.createObject("lng_element"
                                , streamnr=self.parentRecord.streamnr
                                , seqnr=seqnr
                                , parent_elementnr=self.parentRecord.elementnr
                                , text=element
                                , languagenr=self.parentRecord.languagenr
                                , elementtypecode=self.cmbSubType.currentKey()
                                , usernr=Config.usernr
                                )
        record.insert()
        seqnr+=1
    self.refresh() 
  
  def slotItemSelected(self, item):
    self.dlgOpen=dlgElement(Config.app, self, 'Edit Morpheme', "Morpheme", 
                 self.item_to_record[item],
                 UPDATE, Config.app.tables["lng_element"])
    self.connect(self.dlgOpen, PYSIGNAL("sigAcceptData")
                             , self.slotOpenAccept)
    self.dlgOpen.show()

  def slotOpenAccept(self):
    rec=dlgOpen.getMasterRecord()
    try:
      self.item_to_record[item]=rec
    except KeyError:
      QMessageBox.information(self, "Kura Error",
          "Cannot update the text in the view.")
    item.setText(0, rec.text)

class tabElementLexeme(tabLexeme): 
  
  def __init__(self, parent, parentRecord):
    tabLexeme.__init__(self, parent)
    self.parentRecord=parentRecord
    if self.parentRecord.lexnr != None:
      self.lexeme=Config.app.getObject( "lng_lex"
                                      , lexnr=self.parentRecord.lexnr
                                      )
      self.__setValues(self.lexeme)
      
    self.connect(self.bnZoom, SIGNAL("clicked()"), self.__zoom)
    self.connect(self.bnPick, SIGNAL("clicked()"), self.__pick)
    self.connect(self.bnAdd, SIGNAL("clicked()"), self.__add)

  def __setValues(self, lexeme):
    self.txtForm.setFont(Config.font)
    self.txtPhoneticForm.setFont(Config.font)
    self.txtGlosse.setFont(Config.font)
    self.txtDescription.setFont(Config.font)
    self.txtLanguage.setFont(Config.font)

    self.txtForm.setText(lexeme.getFieldValue("form"))
    self.txtPhoneticForm.setText(lexeme.getFieldValue("phonetic_form"))
    self.txtGlosse.setText(lexeme.getFieldValue("glosse"))
    self.txtDescription.setText(lexeme.getFieldValue("description"))
    self.txtLanguage.setText(lexeme.getFieldValue("language"))

  def __zoom(self):
    self.dlgZoom=dlgLexeme(Config.app, self, 'Edit lexical item', "Lexeme", 
                  self.lexeme,  UPDATE, self.lexeme.tableDef)
    self.connect(self.dlgZoom, PYSIGNAL("sigAcceptData")
                             , self.slotZoomAccept)
    self.dlgZoom.show()
      
  def slotZoomAccept(self):
    self.lexeme=self.dlgOpen.getMasterRecord()
    self.parentRecord.lexnr=self.lexeme.lexnr
    self.__setValues(self.lexeme)
   
  def __pick(self):
    #
    # This is a modal dialog
    #
    dlgPick=dlgLexChooser(self, self.parentRecord)
    dlgPick.txtForm.setText(self.parentRecord.getFieldValue("text"))
    dlgPick.refreshSource()
    if dlgPick.exec_loop()==1:
      self.lexeme=dlgPick.masterRecord
      self.parentRecord.lexnr=self.lexeme.lexnr
      self.__setValues(self.lexeme)


  def __add(self):
      if QMessageBox.warning(self, "Kura"
         , "Do you want to add this element to the lexicon?"
         , "Yes", "No", "Cancel"
         , 2, 3
         ) == 0:
         
        self.lexeme=Config.app.createObject("lng_lex"
           , languagenr=self.parentRecord.languagenr
           , form=self.parentRecord.text
           , usernr=Config.usernr
           , phonetic_form=self.parentRecord.getPhoneticTranscription()
           , glosse=self.parentRecord.getGlosse()
           , description=self.parentRecord.getDescription()
           )
           
        if self.lexeme.glosse is None:
          self.lexeme.glosse="<empty>"
          
        self.lexeme.insert()
        self.parentRecord.lexnr=self.lexeme.lexnr
        self.__setValues(self.lexeme)

class dlgElement(guiTabDialog):

  def __init__(self, app, parent, title, firstTabTitle, 
            record, mode, tableDef, showChildren=FALSE):
            
    guiTabDialog.__init__( self
                         , app=app
                         , parent=parent
                         , title=title
                         , firstTabTitle="&Elements"
                         , record=record
                         , mode=mode
                         , tableDef=tableDef
                         , showChildren=FALSE
                         , addBottomSpring=TRUE)

    self.tagstab=tabElementTags(self, app, record)
    guiTabDialog.addChildTab( self
                            , "&Tags"
                            , self.tagstab
                            , record
                            , DETAIL)

    guiTabDialog.addChildTab(self
                            , "&Related Lexeme"
                            , tabElementLexeme(self, record)
                            , record
                            , DETAIL)
                            
    guiTabDialog.addChildTab( self
                            , "&Morphemes"
                            , tabElementSplit(self, record)
                            , record
                            , DETAIL)
  def accept(self):
    try:
      self.tagstab.frmElementTags.save()
    except Exception, e:
      print str(e)
      pass  
    guiTabDialog.accept(self)
    
def main():

  qapp=QApplication(sys.argv)
  toplevel=guiListView( Config.app
                      , tableName="lng_element"
                      , searchRec=Config.app.createObject('lng_element', elementnr="6")
                      , dlgForm=dlgElement
                      , dlgSearch=None
                      , parent=None
                      )
  toplevel.show()
  qapp.connect(qapp, SIGNAL('lastWindowClosed()'), qapp, SLOT('quit()'))
  qapp.exec_loop()
  
if __name__=="__main__":
  if len(sys.argv)>1:
    if sys.argv[1]=='-p':
      import profile
      profile.run('main()')
  else:
    main()

__copyright__="""
/***************************************************************************
    copyright            : (C) 2000 by Boudewijn Rempt 
                           see copyright notice for license
    email                : boud@rempt.xs4all.nl
    Revision             : $Revision: 1.14 $
    Last edited          : $Date: 2001/01/30 20:27:57 $
    
    CVS Log:         
    $Log: dlgelement.py,v $
    Revision 1.14  2001/01/30 20:27:57  boud
    Unsaved added tags are now saved on OK.

    Revision 1.13  2001/01/30 20:04:42  boud
    Removed print statements.

    Revision 1.12  2001/01/30 19:43:02  boud
    Several small bugfixes.

    Revision 1.11  2001/01/30 18:10:08  boud
    Made dialogs non-modal, added statusbar, added charmap window.

    Revision 1.10  2001/01/26 20:02:17  boud
    Move to dialog modelessness.

    Revision 1.9  2001/01/15 20:21:09  boud
    Added import/export filters, bugfixes.

    Revision 1.8  2001/01/08 20:55:06  boud
    Cleanup for version 1.0

 ***************************************************************************/
"""
