import sys
from qt import *
from kuralib.config import Config

TRUE=1
FALSE=0

class comboProxy(QObject):
  """
  This class proxies for a standard QComboBox to provide 
  a picklist filled from the database. This class does
  not use the record.picklist functionality, since it
  is intended for situations where there isn't a record.
  """
  def __init__(self, comboBox, tableName, currentKey=None, nullAllowed=FALSE):
    QObject.__init__(self)
    self.comboBox=comboBox
    self.comboBox.setFont(Config.font)

    self.key_to_index={}
    self.index_to_key={}
    
    if nullAllowed==TRUE:
      self.comboBox.insertItem("<None>")
      self.key_to_index[None]=self.comboBox.count()-1
      self.index_to_key[self.comboBox.count()-1]=None
      
    for row in Config.app.getObjects(tableName):
      self.comboBox.insertItem(row.getDescription())
      primaryKey=row.getPrimaryKey()
      self.key_to_index[primaryKey]=self.comboBox.count()-1
      self.index_to_key[self.comboBox.count()-1]=primaryKey
         
    if currentKey!=None:
      self.setCurrentItem(currentKey)

    self.connect(self.comboBox, SIGNAL("activated (int)")
                              , self.slotItemSelected
                              )
    self.comboBox.setAutoCompletion(TRUE)
   
  def currentText(self):
    return unicode(self.comboBox.currentText())
    
  def currentKey(self):
    try:
      return self.index_to_key[self.comboBox.currentItem()]
    except KeyError:
      return None

  def setCurrentItem(self, key):
    if self.key_to_index.has_key(key):
      self.comboBox.setCurrentItem(self.key_to_index[key])

  def slotItemSelected(self, index):
    try:
      rec=self.index_to_key[index]
      self.emit(PYSIGNAL("recSelected"),(rec, index))
    except KeyError:
      pass
      
   

__copyright__="""
/***************************************************************************
    copyright            : (C) 2000 by Boudewijn Rempt 
                           see copyright notice for license
    email                : boud@rempt.xs4all.nl
    Revision             : $Revision: 1.7 $
    Last edited          : $Date: 2001/01/30 20:04:42 $
    
    CVS Log:         
    $Log: kuracomboproxy.py,v $
    Revision 1.7  2001/01/30 20:04:42  boud
    Removed print statements.

    Revision 1.6  2001/01/26 20:02:17  boud
    Move to dialog modelessness.

    Revision 1.5  2001/01/08 20:55:06  boud
    Cleanup for version 1.0

 ***************************************************************************/
"""
