import sys
import string

from qt import *
TRUE=1
FALSE=0
try:
  from kuralib.config import Config
except:
  sys.path.append("..")
  from kuralib.config import Config
from dbobj.dbexceptions import dbRecordNotFoundException

class kuraIlCanvas(QCanvas):
  """
  FIXME: The layout of an interlinear text is now more-or-less fixed.
  A nice xml-based template would be excellent for this.
  """
  tagDefinitions={}
  elementTypeCodes={}
  
  def __init__(self, parent=0, name=0):
    QCanvas.__init__(self, parent, name)

    self.currentY=10
    self.lineHeight=QFontMetrics(Config.font).height()
    self.lineSpacing=QFontMetrics(QFont(Config.font)).lineSpacing()
    self.lineSetSpacing= 5
    self.elementSpacing=8
    self.leftMargin=50
    self.rightMargin=20
    self.currentX=self.leftMargin
    self.maxLineWidth=500
    self.width=self.maxLineWidth
    self.lines=[]
    self.normalfont=QFont(Config.font.family(), Config.font.pointSize(), QFont.Normal, FALSE, QFont.Unicode)
    self.italicfont=QFont(Config.font.family(), Config.font.pointSize(), QFont.Normal, TRUE, QFont.Unicode)
    self.boldfont=QFont(Config.font.family(), Config.font.pointSize(), QFont.Bold, FALSE, QFont.Unicode)
    self.bolditalicfont=QFont(Config.font.family(), Config.font.pointSize(), QFont.Bold, TRUE, QFont.Unicode)
    self.titlefont=QFont(Config.font.family(), Config.font.pointSize() * 2, QFont.Bold, TRUE, QFont.Unicode)

    self.item_index=[]
    self.item_to_record={}
    self.record_to_item={}

    self.styles={}
    
    self.marginItems=[]
    
    for style in Config.app.getObjects("lng_stylecode"):
      style.qfont=QFont(style.font, int(style.size), int(style.weight), int(style.italic), int(style.charset))
        
      self.styles[style.stylecode]=style

    self.defaultStyle=Config.app.createDefaultObject("lng_stylecode")
    self.defaultStyle.qfont=self.normalfont
    
    
  def setItemText(self, record, text):
    try:
      item=self.record_to_item[record][0]
    except KeyError:
      pass
    else:
      item.setText(text)
      
  def getTagStyle(self, tagRecord):
    if tagRecord==None:
      return self.defaultStyle
    else:
      tagDefinition=self.__cache(kuraIlCanvas.tagDefinitions,tagRecord.tag, "lng_tag", tag=tagRecord.tag)
      style=self.styles.get(tagDefinition.stylecode, self.defaultStyle)
      return style

  def getElementStyle(self, element):
    elementTypeCode=self.__cache(kuraIlCanvas.elementTypeCodes, element.elementtypecode, "lng_elementtypecode", elementtypecode=element.elementtypecode)
    style=self.styles.get(elementTypeCode.stylecode, self.defaultStyle)
    return style
  
  def __cache(self, cache, key, tableName, **args):
    if cache.has_key(key):
      return cache[key]
    else:
      try:
        rec=Config.app.getObject(tableName, fields=args)
        cache[key]=rec
        return rec
      except dbRecordNotFoundException, errorMessage:
        print "Configuration Error: key " + key + " does not exist in " + tableName +"."
        raise

  def mapRecord(self, record, item):
    self.item_index.append(item)
    self.item_to_record[item]=record
    if self.record_to_item.has_key(record):
      self.record_to_item[record].append(item)
    else:
      self.record_to_item[record]=[item]

  def newLine(self, lineSpacing=None):
    if lineSpacing==None:
      self.currentY=self.currentY + self.lineSpacing 
      self.currentX=self.leftMargin
    else:
      self.currentY=self.currentY + lineSpacing
      self.currentX=self.leftMargin

  def marginItem(self, item, y=None):
    if y==None:
      y=self.currentY
    item.setX(self.leftMargin - (item.boundingRect().width() + 10))
    item.setY(y)
    item.show()
      
  def centerItem(self, item):
    if item.boundingRect().width() < (self.maxLineWidth - self.leftMargin):
      item.setX(self.leftMargin + ((self.maxLineWidth - self.leftMargin) - item.boundingRect().width()) / 2)
      item.setY(self.currentY)
      item.show()
    else:
      item.setX(self.leftMargin)
      item.setY(self.currentY)
      self.width=self.leftMargin + item.boundingRect().width()
      item.show()

  def placeItem(self, item, spacing=TRUE):
    w=item.boundingRect()
    #
    # Item at beginning of line, but it is too long
    #
    if self.currentX == self.leftMargin and self.currentX + w.width() > self.maxLineWidth:
      item.setX(self.currentX)
      item.setY(self.currentY)
      item.show()
      self.width=self.currentX + w.width() + self.rightMargin
      self.newLine()
      return
      
    #
    # Move to new line
    #
    if self.currentX + w.width() > self.maxLineWidth:
      
      self.newLine()
 
    item.setX(self.currentX)
    item.setY(self.currentY)

    if spacing==TRUE:
      self.currentX=self.currentX + w.width() + self.elementSpacing
    else:
      self.currentX=self.currentX + w.width()

    item.show()
    
  def placeSet(self, itemSet, w):
    y=self.currentY
    if self.currentX + w > self.maxLineWidth:
      self.newLine(len(itemSet) * self.lineSpacing)
      #self.writeLineMarkers(self.currentY)
      y=self.currentY

    for subSet in itemSet:
      x=self.currentX
      for item in subSet:
        item.setX(x)
        x=x+item.boundingRect().width()
        item.setY(y)
        item.show()
      y=y+self.lineSpacing
      
    self.currentX=self.currentX + w + self.elementSpacing
    return len(itemSet) * self.lineSpacing
    
  def writeLineMarkers(self, y):
    for itemDef in Config.InterLinearElementPresentation.split():
      k,v=itemDef.split(".")
      if k=="element_tag":
        item=QCanvasText(v, self.normalfont, self)
        self.mapRecord(None, item)
        self.marginItem(item, y)
      y=y+self.lineHeight

    
  def handleText(self, lngText):
    # Text
    for itemDef in Config.InterLinearTextPresentation.split():
      base, field=itemDef.split(".")
      if base=="text":
        if field=="title":
          item=QCanvasText(lngText.getFieldValue(field), self.titlefont, self)
          self.centerItem(item)
          self.newLine(item.boundingRect().height() + self.lineSpacing)
          self.mapRecord(lngText, item)

        else:
          value=lngText.getFieldValue(field)
          if value!=None:
            for word in value.split():
              item=QCanvasText(word + " ", self.normalfont, self)
              self.mapRecord(lngText, item)
              self.placeItem(item, FALSE)
            self.newLine(self.lineSpacing + self.lineSetSpacing)
          
      elif base=="text_tag":
        tagRecord=lngText.getTag(field)
        style=self.getTagStyle(tagRecord)
        text=tagRecord.getDescription(style.showifnull)
        if int(style.wordwrap)==TRUE:
          for line in text.split("\n"):
            for word in line.split():
              item=QCanvasText(word + " ", style.qfont, self)
              self.mapRecord(tagRecord, item)
              self.placeItem(item, FALSE)
            self.newLine()
        else:
          for line in text.split("\n"):
            item=QCanvasText(line + " ", style.qfont, self)
            self.mapRecord(tagRecord, item)
            self.placeItem(item, FALSE)
        self.newLine(self.lineSpacing + self.lineSetSpacing)

      elif base=="stream":
        for stream in lngText.getStreams():
          self.handleStream(stream)

  def handleSubElements(self, parentelement, w):
    set=[]
    els=parentelement.getElements()
    
    for itemDef in Config.InterLinearElementPresentation.split():
      k,v=itemDef.split(".")
      if k=="element":
        t=[] # temporary array of QCanvasText items
        tw=0 # width of the temporary array of QCanvasText item
        for element in els:
          style=self.getElementStyle(element)
          item=QCanvasText(element.getFieldValue("text") + 
              kuraIlCanvas.elementTypeCodes[element.elementtypecode].joinmarker
              , style.qfont, self)
          self.mapRecord(element, item)
          t.append(item)
          tw=tw+item.boundingRect().width()
        if len(t) > 0:
          # remove the last joinmark
          last=len(t)-1
          qs=t[last].text()
          text=unicode(qs)
          t[last].setText(text[:-1])
          # update width
        if tw > w:
          w=tw
        set.append(t)
      elif k=="element_tag":
        t=[] # temporary array of QCanvasText items
        tw=0 # width of the temporary array of QCanvasText item
        for element in els:
          tagRecord=element.getTag(v)
          style=self.getTagStyle(tagRecord)
          item=QCanvasText(tagRecord.getDescription(style.showifnull) + 
               kuraIlCanvas.elementTypeCodes[element.elementtypecode].joinmarker
               , style.qfont, self)
          self.mapRecord(tagRecord, item)
          t.append(item)
          tw=tw+item.boundingRect().width()
        if len(t) > 0:
          # remove the last joinmark
          last=len(t)-1
          qs=t[last].text()
          text=unicode(qs)
          t[last].setText(text[:-1])
        # update width
        if tw > w:
          w=tw
        set.append(t)  
      # FIXME: this is not fully recursive yet
      elif k=="element_element":
        pass
      
    return set, w

  def handleElement(self, element):
    set=[]
    w=0
    for itemDef in Config.InterLinearElementPresentation.split():
      k,v=itemDef.split(".")
      if k=="element":
        style=self.getElementStyle(element)
        item=QCanvasText(element.getFieldValue(v), style.qfont, self)
        self.mapRecord(element, item)
        set.append([item])
        if item.boundingRect().width() > w:
          w=item.boundingRect().width()
          
      elif k=="element_tag":
        tagRecord=element.getTag(v)
        style=self.getTagStyle(tagRecord)
        item=QCanvasText(tagRecord.getDescription(style.showifnull), style.qfont, self)
        self.mapRecord(tagRecord, item)
        set.append([item])
        if item.boundingRect().width() > w:
          w=item.boundingRect().width()
          
      elif k=="element_element":
        subset, w = self.handleSubElements(element, w)
        set=set + subset
    
    h=self.placeSet(set, w)
    return h

  def handleStream(self, stream):
    item=QCanvasText(str(int(stream.seqnr) + 1), self.boldfont, self)
    self.mapRecord(stream, item)
    self.marginItem(item)

    for streamDef in Config.InterlinearStreamPresentation.split():
      base, field=streamDef.split(".")
      elementSetHeight=0
      if base=="element":
        if field=="element":
          #self.writeLineMarkers(self.currentY)
          for element in stream.getElements():
            if element.parent_elementnr == None or element.parent_elementnr == 0:
              elementSetHeight=self.handleElement(element)
          self.newLine(elementSetHeight)
           
      elif base=="stream":
        for word in stream.text.split():
          item=QCanvasText(word + " ", self.italicfont, self)
          self.mapRecord(stream, item)
          self.placeItem(item, FALSE)
        self.newLine()

      elif base=="stream_tag":
        tagRecord=stream.getTag(field)
        style=self.getTagStyle(tagRecord)
        text=tagRecord.getDescription(style.showifnull)
        if int(style.wordwrap)==TRUE:
          for line in text.split("\n"):
            for word in line.split():
              item=QCanvasText(word + " ", style.qfont, self)
              self.mapRecord(tagRecord, item)
              self.placeItem(item, FALSE)
            self.newLine()
        else:
          for line in text.split("\n"):
            item=QCanvasText(line + " ", style.qfont, self)
            self.mapRecord(tagRecord, item)
            self.placeItem(item, FALSE)
        self.newLine()
      self.newLine(self.lineSetSpacing)

  def setText(self, lngText):

    self.lngText=lngText
    
    self.handleText(lngText)
    
    self.newLine()

    self.scol = QCanvasLine(self)
    self.scol.setPoints(self.leftMargin - 5, 0,
                        self.leftMargin - 5, self.currentY
                       )
    self.scol.show()

    self.resize(self.width, self.currentY)
    self.update()
    kuraIlCanvas.tagDefinitions={}


__copyright__="""
/***************************************************************************
    copyright            : (C) 2000 by Boudewijn Rempt 
                           see copyright notice for license
    email                : boud@rempt.xs4all.nl
    Revision             : $Revision: 1.12 $
    Last edited          : $Date: 2001/01/30 18:10:08 $
    
    CVS Log:         
    $Log: kurailcanvas.py,v $
    Revision 1.12  2001/01/30 18:10:08  boud
    Made dialogs non-modal, added statusbar, added charmap window.

    Revision 1.11  2001/01/25 23:04:15  boud
    Added exception.

    Revision 1.10  2001/01/25 23:03:36  boud
    Exception added.

    Revision 1.9  2001/01/24 19:18:47  boud
    Made more robust.

    Revision 1.8  2001/01/24 19:15:32  boud
    UI changes.

    Revision 1.7  2001/01/15 20:21:09  boud
    Added import/export filters, bugfixes.

    Revision 1.6  2001/01/08 20:55:06  boud
    Cleanup for version 1.0

 ***************************************************************************/
"""
