from qt                   import *

from kuralib.config       import Config
from kuralib.lng_strm     import lng_stream
from kuralib.lng_sttg     import lng_stream_tag
from kuralib.lng_elmt     import lng_element
from kuralib.lng_eltg     import lng_element_tag
from kuralib.lng_text     import lng_text
from kuralib.lng_txtg     import lng_text_tag

from kuragui.guitabdialog import guiTabDialog
from kuragui.constants    import *
from kuragui.guiform      import guiForm

from resource             import TRUE, FALSE
from kurailcanvas         import kuraIlCanvas
from kurailcanvasview     import kuraIlCanvasView
from kuratextselector     import kuraTextSelector
from dlgtext              import dlgText
from dlgstream            import dlgStream
from dlgelement           import dlgElement
from dlgtag               import dlgTag
from dlgnewtext           import dlgNewText
     
class kuraTextListView(QSplitter):

  def __init__(self, parent, searchRec):
    QSplitter.__init__(self, parent)
    self.canvas=kuraIlCanvas()
    
    self.textSelector=kuraTextSelector(searchRec, self)
    self.canvasView=kuraIlCanvasView(self.canvas, self)
    self.setResizeMode(self.textSelector, QSplitter.KeepSize)
    
    self.connect(self.textSelector, PYSIGNAL("sigRecordSelected"), 
                                    self.slotCanvasText)
    self.connect(self.textSelector, PYSIGNAL("sigFindRecord"), 
                                    self.Find)
    self.connect(self.textSelector, PYSIGNAL("sigCreateNewItem"), 
                                    self.slotNewText)
    self.connect(self.textSelector, PYSIGNAL("sigItemDeleted"), 
                                    self.slotDeleteText)
    
    self.connect(self.canvasView, PYSIGNAL("sigMousePressedOn"), 
                                  self.slotCanvasItemSelected)
    self.connect(self.canvasView, PYSIGNAL("sigDoubleClickOn"), 
                                  self.slotCanvasItemOpened)
    self.connect(self.canvasView, PYSIGNAL("sigEnterPressed"), 
                                  self.slotCanvasItemSelected)
        
  def slotCanvasText(self, text, lvi):
    self.parent().setCursor(Qt.waitCursor)  
    self.textSelector.setCursor(Qt.waitCursor)  
    
    self.canvas=kuraIlCanvas()
    self.canvas.setText(text)
    self.canvasView.setCanvas(self.canvas)
    self.canvasView.setFocus()
    
    self.parent().setCursor(Qt.arrowCursor)
    self.textSelector.setCursor(Qt.arrowCursor)  

  def __openText(self, item, record):
    self.dlgOpen=dlgText(app=Config.app, parent=self, title='Edit Text'
                   , firstTabTitle="Text", record=record
                   , mode= UPDATE, tableDef=Config.app.tables["lng_text"])
    self.connect(self.dlgOpen, PYSIGNAL("sigAcceptData")
                             , self.slotOpenTextAccept
                             )
    self.dlgOpen.show()
    
  def slotOpenTextAccept(self):
    rec=self.dlgOpen.getMasterRecord()
    self.canvas.setItemText(rec, rec.title)
    self.canvas.update()
 
  def __openStream(self, item, record):
    self.dlgOpen=dlgStream(Config.app, self, 'Edit Stream', "Stream", 
                 record,  UPDATE, Config.app.tables["lng_stream"])
    self.connect(self.dlgOpen, PYSIGNAL("sigAcceptData")
                             , self.slotOpenStreamAccept
                             )
    self.dlgOpen.show()
    
  def slotOpenStreamAccept(self):
    rec=self.dlgOpen.getMasterRecord()
    self.canvas.update()

  def __openElement(self, item, record):
    self.dlgOpen=dlgElement(Config.app, self, 'Edit Element', "Element", 
                 record,  UPDATE, Config.app.tables["lng_element"])
    self.connect(self.dlgOpen, PYSIGNAL("sigAcceptData")
                             , self.slotOpenElementAccept
                             )
    self.dlgOpen.show()

  def slotOpenElementAccept(self):
    rec=self.dlgOpen.getMasterRecord()
    self.canvas.setItemText(rec, rec.text)
    self.canvas.update()

  def __openTag(self, item, record, title):
    self.dlgOpen=dlgTag( parent=self
                  , title=title
                  , parentTableName=record.table
                  , record=record
                  )
    self.connect(self.dlgOpen, PYSIGNAL("sigAcceptData")
                             , self.openTagAccept
                             )
    self.dlgOpen.show()
    
  def openTagAccept(self):
    rec=self.dlgOpen.getMasterRecord()
    self.canvas.setItemText(rec, rec.getDescription())
    self.canvas.update()
    
  def slotCanvasItemSelected(self, item, record): pass

  def slotCanvasItemOpened(self, item, record):
    if isinstance(record, lng_stream):    
       self.__openStream(item, record)
    elif isinstance(record, lng_text):
       self.__openText(item, record)
    elif isinstance(record, lng_element):
       self.__openElement(item, record)
    elif isinstance(record, lng_element_tag):
      self.__openTag(item, record, "Edit word tag " + record.tag)
    elif isinstance(record, lng_stream_tag):
      self.__openTag(item, record, "Edit sentence tag " + record.tag)
    elif isinstance(record, lng_text_tag):
      self.__openTag(item, record, "Edit text tag " + record.tag)
      
    else:
      pass


  def refresh(self, searchRec):
    self.setCursor(Qt.waitCursor)
    self.textSelector.refresh(searchRec)
    self.canvas=kuraIlCanvas()
    self.canvasView.setCanvas(self.canvas)
    self.setCursor(Qt.arrowCursor)  
        
  def Find(self):
    if Config.textSortCriterion == "filter":
      if Config.useDefaultForSearch==TRUE:
        record=Config.app.createDefaultObject("lng_text")
      else:
        record=Config.app.createObject("lng_text")

      self.dlgSearch=guiTabDialog(Config.app, self, "Find", "Text"
          , record, SEARCH, Config.app.tables["lng_text"], showChildren=FALSE)
      self.connect(self.dlgSearch, PYSIGNAL("sigAcceptData")
                                 , self.slotFindAccept
                                 )
      self.dlgSearch.show()
    else:
      QMessageBox.information(self, "Kura", 
            "You can only search when in filter mode.")

  def slotFindAccept(self):
    self.refresh( dlgSearch.getMasterRecord() )
   
  def New(self):
    self.canvas=kuraIlCanvas()
    self.textSelector.slotNewItem()
  
  def slotNewText(self, defaultRec):
    self.dlgNew=dlgNewText(self)
    self.connect(self.dlgNew, PYSIGNAL("sigAcceptData")
                            , self.slotNewTextAccept
                            )
    self.dlgNew.show()

  def slotNewTextAccept(self):
    record=Config.app.getObject("lng_text", textnr=self.dlgNew.recText.textnr)
    self.textSelector.slotAddItem(record)
  
  def Open(self):
    try:    
      record, lvi=self.textSelector.currentRecord()
    except TypeError:
      # nothing selected
      return
      
    
    self.dlgOpen=dlgText(Config.app, self, 'Edit Text', "Text", 
                   record,  UPDATE, Config.app.tables["lng_text"])
    self.connect(self.dlgOpen, PYSIGNAL("sigAcceptData")
                             , self.slotOpenAccept
                             )
    self.dlgOpen.show()
    
  def slotOpenAccept(self):
    rec=self.dlgOpen.getMasterRecord()
    self.textSelector.slotUpdateItem(rec, lvi)
    
  def Delete(self):
    self.textSelector.slotDeleteItem()
    
  def slotDeleteText(self):
    self.canvas=kuraIlCanvas()
    self.canvasView.setCanvas(self.canvas)

  def keyPressEvent(self, e):
    if e.key()==Qt.Key_F2:
      if Config.textSortCriterion=="filter":
        self.Find()

__copyright__="""
/***************************************************************************
    copyright            : (C) 2000 by Boudewijn Rempt 
                           see copyright notice for license
    email                : boud@rempt.xs4all.nl
    Revision             : $Revision: 1.14 $
    Last edited          : $Date: 2001/02/01 19:29:44 $
    
    CVS Log:         
    $Log: kuratextlistview.py,v $
    Revision 1.14  2001/02/01 19:29:44  boud
    Restored opening of texts.

    Revision 1.13  2001/01/31 20:04:36  boud
    Bugfixes.

    Revision 1.12  2001/01/30 20:04:42  boud
    Removed print statements.

    Revision 1.11  2001/01/30 18:10:08  boud
    Made dialogs non-modal, added statusbar, added charmap window.

    Revision 1.10  2001/01/26 20:02:17  boud
    Move to dialog modelessness.

    Revision 1.9  2001/01/24 19:18:47  boud
    Made more robust.

    Revision 1.8  2001/01/15 20:21:09  boud
    Added import/export filters, bugfixes.

    Revision 1.7  2001/01/08 20:55:06  boud
    Cleanup for version 1.0

 ***************************************************************************/
"""
