from qt import *
from kuralib.config import Config
from kuradom import KuraDOM

class KuraDBWriter(QObject):

  def __init__(self, dom, *args):
    apply(QObject.__init__,(self,)+args)
    self._dom=dom
    self.elementDepth=[]
    self.elementSequence=[0]
    self.currentTextnr=None
    self.currentStreamnr=None
    self.strm_seqnr=0
    
    self.connect(self._dom, PYSIGNAL("sigTextStarted"), self._saveText)
    self.connect(self._dom, PYSIGNAL("sigStreamStarted"), self._saveStream)
    self.connect(self._dom, PYSIGNAL("sigStreamEnded"), self._endStream)
    self.connect(self._dom, PYSIGNAL("sigElementStarted"), self._saveElement)
    self.connect(self._dom, PYSIGNAL("sigElementEnded"), self._elementEnded)
    self.connect(self._dom, PYSIGNAL("sigTextTag"), self._saveTextTag)
    self.connect(self._dom, PYSIGNAL("sigStreamTag"), self._saveStreamTag)
    self.connect(self._dom, PYSIGNAL("sigElementTag"), self._saveElementTag)
    
  def _saveText(self, text):
    text.insert()
    self.currentTextnr=text.textnr

  def _saveTextTag(self, tag):
    tag.textnr=textnr
    tag.insert()

  def _saveStream(self, stream):
    stream.textnr=self.currentTextnr
    stream.seqnr=self.strm_seqnr
    stream.insert()
    self.currentStreamnr=stream.streamnr
    
  def _endStream(self, stream):
    self.strm_seqnr+=1
    self.elementDepth=[]
    self.elementSequence=[0]
    
  def _saveStreamTag(self, tag):
    tag.streamnr=self.currentStreamnr
    tag.insert()
    
  def _saveElement(self, element):
    element.streamnr=self.currentStreamnr
    if len(self.elementDepth) > 0:
      element.parent_elementnr=self.elementDepth[-1]
    
    element.seqnr=self.elementSequence[-1]
    
    element.insert()
    
    self.elementSequence[-1]+=1
    self.elementDepth.append(element.elementnr)
    self.elementSequence.append(0)
    
  def _elementEnded(self, element):
    del self.elementDepth[-1]
    del self.elementSequence[-1]
    
  def _saveElementTag(self, tag):    
    tag.elementnr=self.elementDepth[-1]
    tag.insert()

  def start(self):
    self._dom.walkTree()
