import sys
from qt import *
from kuralib.config import Config
from kuradom import KuraDOM
from kuradbreader import KuraDBReader

def nvl(item):
  if item==None:
    return "#"
  else:
    return item

class KuraPlaintextWriter(QObject):
  
  def __init__(self, dom, *args):
    apply(QObject.__init__,(self,)+args)
    self.dom=dom
    self.indent=0
    self.maxDepth=0
    self.elementDepth=[]
    
    self.currentTextnr=None
    self.currentStreamnr=None
    self.currentElementnr=None
    
    self.connect(self.dom, PYSIGNAL("sigTextStarted"), self._startText)
    self.connect(self.dom, PYSIGNAL("sigTextStarted"), self._textEnded)
    self.connect(self.dom, PYSIGNAL("sigStreamStarted"), self._startStream)
    self.connect(self.dom, PYSIGNAL("sigStreamStarted"), self._streamEnded)
    self.connect(self.dom, PYSIGNAL("sigElementStarted"), self._startElement)
    self.connect(self.dom, PYSIGNAL("sigElementEnded"), self._elementEnded)
    
    self.connect(self.dom, PYSIGNAL("sigTextTag"), self._startTag)
    self.connect(self.dom, PYSIGNAL("sigStreamTag"), self._startTag)
    self.connect(self.dom, PYSIGNAL("sigElementTag"), self._startTag)


  def __printRecord(self, rec):
    print self.indent * "\t", "#", rec.table
    for field in rec.fieldOrder:
      print self.indent * "\t", "\\" + field, "=", rec.getFieldValue(field)
    print
    
  def _startText(self, text):
    self.currentTextnr=text.textnr
    self.indent=0
    self.__printRecord(text)

  def _textEnded(self, text): 
    self.indent-=1

  def _startStream(self, stream):
    self.indent+=1
    self.currentStreamnr=stream.streamnr
    self.__printRecord(stream)
        
  def _streamEnded(self, stream):
    self.indent-=1
   
  def _startElement(self, element):
    self.indent+=1
    self.elementDepth.append(element.elementnr)
    self.currentElementnr=element.elementnr
    self.__printRecord(element)
      
  def _elementEnded(self, element):
    del self.elementDepth[len(self.elementDepth)-1]
    self.currentElementnr=self.elementDepth[-1:]
    self.indent-=1 
    
  def _startTag(self, tag):
    self.indent+=1
    self.__printRecord(tag)
    self.indent-=1
    
  
  def start(self):
    print r"\collection"
    print r"\version 1.0"
    print
    self.dom.walkTree()

    
    
def main(args):
  dbr=KuraDBReader()
  for arg in args:
    dbr.start(arg)
  xmlsaver=KuraPlaintextWriter(dbr.getDom())
  
  xmlsaver.start()
  
if __name__=="__main__":
  main(sys.argv[1:])
