#!/usr/bin/env python

import xmllib
import sys
import string

from kuralib.config import Config
from kuradom import KuraDOM
from kurailwriter import KuraILWriter

TRUE=1
FALSE=0

INTEXT=1
INSTREAM=2
INLINE=3

class KuraXMLImport(xmllib.XMLParser): 
  """
  Reads simple XML tagged files:

<?xml version="1.0" encoding='iso-8859-1'?>
<kura_text 
  title="The Charter of the Guild of Anshen" languagenr="13"
  description="This is purported to have been written by Vauri,
  Commander of the Order of the Sworn of Anshen in the time of King
  Vegelin II, the Great."
>
<stream>
  <line
    text="shen vestyn anin"
    gloss="light-nom-s source-nom-s Poss1p"
  />
  <translation text="The Light is our source."/>
</stream>
</kura_text>

The languagenr should exist; there should be a tag "GL" and a tag "TR". There
should be an elementtypecode "FORM" Do not confuse this with kuraxmlreader:
that reads  in date written out by kuraxmlwriter, a complete dump of a text
from the database. This is to be used _before_ the text is in  the
database.  

  """
  def __init__(self):
    xmllib.XMLParser.__init__(self)
    self._dom=KuraDOM()
    self.state=0
    self.currentText=None
    self.currentStream=None
    

    
    self.elements={'kura_text'      : (self.start_kura_text, self.end_kura_text)
                  ,'stream'         : (self.start_stream, self.end_stream)
                  ,'line'           : (self.start_line, self.end_line)
                  ,'note'           : (self.start_note, self.end_note)
                  ,'translation'    : (self.start_translation, self.end_translation)
                  }


  def getDom(self):
    return self._dom
    
  def doReset(self):
    xmllib.XMLParser.reset(self)
    self.currentText=None
    self.currentStream=None
        
  def start_kura_text(self, attrs):
    self.currentText=self._dom.addText(attrs)
    self.state=INTEXT
    
  def end_kura_text(self):
    self.state=0

  def start_stream(self, attrs):
    self.state=INSTREAM
    attrs["languagenr"]=attrs.get('languagenr', self.currentText.languagenr)
    self.currentStream=self._dom.addStream(attrs)
    
  def end_stream(self):
    self.state=INTEXT
    
  def start_line(self, attrs):
    self.state=INLINE
    if attrs.has_key('text'):
      self.currentStream.text=string.join(string.split(attrs['text']))
      elements=string.split(attrs['text'])
      glosses=string.split(attrs['gloss'])
      if len(elements)<> len(glosses):
        raise "Elements and glosses don't match!"
      for element, glosse in map(None, elements, glosses):
        e=self._dom.addElement({ "text":element
                               , "languagenr":self.currentStream.languagenr
                               , "elementtypecode": "FORM"})
        t=self._dom.addElementTag({ "tag":"GL", "value":glosse})
    
  def end_line(self):
    self.state=INSTREAM
    
  def start_translation(self, attrs):
    if self.state==INSTREAM:
      self._dom.addStreamTag({"tag":"TR", "note":attrs.get("text", "")})
    elif self.state==INTEXT:
      self._dom.addTextTag({"tag":"TR", "note":attrs.get("text", "")})

  def end_translation(self): pass
 
  def start_note(self, attrs):
    if self.state==INSTREAM:
      self._dom.addStreamTag({"tag":"NOTE", "note":attrs.get("text", "")})
    elif self.state==INTEXT:
      self._dom.addTextTag({"tag":"NOTE", "note":attrs.get("text", "")})

  def end_note(self): pass
 
 

def main(args):
  xmlimp=KuraXMLImport()
  for arg in args:
    xmlimp.doReset()
    xmlimp.feed(open(arg).read())
    xmlimp.close()
  ilwr=KuraILWriter(xmlimp.getDom())
  ilwr.start()
  
if __name__=="__main__":
  main(sys.argv[1:])

__copyright__="""
/***************************************************************************
    copyright            : (C) 2000 by Boudewijn Rempt 
                           see copyright notice for license
    email                : boud@rempt.xs4all.nl
    Revision             : $Revision: 1.2 $
    Last edited          : $Date: 2001/01/16 20:58:36 $
    
    CVS Log:         
    $Log: kuraxmlimport.py,v $
    Revision 1.2  2001/01/16 20:58:36  boud
    ...

    Revision 1.1  2001/01/15 20:21:30  boud
    Import/export

    Revision 1.2  2001/01/08 20:55:06  boud
    Cleanup for version 1.0

 ***************************************************************************/
"""
