import sys
import xmllib

from kuralib.config import Config
from kuradom import KuraDOM
from kurailwriter import KuraILWriter

TRUE=1
FALSE=0

class KuraXMLReader(xmllib.XMLParser):

  def __init__(self, *args):
    xmllib.XMLParser.__init__(self)
    self._dom=KuraDOM()

    self.elementDepth=[]
    
    self.tags={}
    for tag in Config.app.getObjects("lng_tag"):
      self.tags[tag.tag]=tag

    self.elementTypes={}
    for elementtypecode in Config.app.getObjects("lng_elementtypecode"):
      self.elementTypes[elementtypecode.elementtypecode]=elementtypecode

    self.elements= {'kuracollection': (self.startCollection, self.endCollection)
                   ,'text': (self.startText, self.endText)
                   ,'text_tag': (self.startTextTag, self.endTextTag)
                   ,'stream': (self.startStream, self.endStream)
                   ,'stream_tag': (self.startStreamTag,self.endStreamTag)
                   ,'element': (self.startElement,self.endElement)
                   ,'element_tag': (self.startElementTag,self.endElementTag)
                   }

  def doReset(self):
    self._dom=KuraDOM()
    xmllib.XMLParser.reset(self)
    self.elementDepth=[]
    
  def startCollection(self, attrs): pass
  
  def endCollection(self): pass
  
  def startText(self, attrs):
    self._dom.addText(attrs)
    
  def endText(self): pass
    
  def startTextTag(self, attrs):
    self._dom.addTextTag(attrs)
  
  def endTextTag(self): pass
  
  def startStream(self, attrs):
    self._dom.addStream(attrs)
  
  def endStream(self): pass
  
  def startStreamTag(self, attrs):
    self._dom.addStreamTag(attrs)
    
  def endStreamTag(self): pass

  def startElement(self, attrs):
    depth=len(self.elementDepth)
    if depth==0:
      self.elementDepth.append(self._dom.addElement(attrs, None))
    else:
      parent=self.elementDepth[depth -1]
      self.elementDepth.append(self._dom.addElement(attrs, parent))
    
  def endElement(self):
    del self.elementDepth[-1:]
    
  def startElementTag(self, attrs):
    self._dom.addElementTag(attrs)
  
  def endElementTag(self): pass
  
  def getDom(self):
    return self._dom
    
  def start(self, text):
    self.elementDepth=[]
    self.doReset()
    xmllib.XMLParser.feed(self, open(text).read())
    xmllib.XMLParser.close(self)

def main(args):
  import time
  t=time.time()
  
  xmlr=KuraXMLReader()
  for arg in args:
    xmlr.start(arg)
  ilwr=KuraILWriter(xmlr.getDom())
  ilwr.start()
  
  print time.time() - t
  
if __name__=="__main__":
  main(sys.argv[1:])
