from constants import TRUE, FALSE, SEARCH
from guiconfig import guiConfig
from qt import *

class guiComboBox(QComboBox):

  def __init__(self, parent, *args):
    QComboBox.__init__(self, FALSE, parent, )
    self.setFont(guiConfig.font)
    self.setAutoCompletion(TRUE)
    self.index_to_key = {}
    self.key_to_index = {}
    self.connect(self, SIGNAL("activated(int)"), self.slotItemSelected)
    self.setSizePolicy(QSizePolicy(7,0,self.sizePolicy().hasHeightForWidth()))
    
  def currentText(self):
    return unicode(QComboBox.currentText(self) )
    
  def insertItem(self, text, index):
    QComboBox.insertItem(self, text)
    self.index_to_key [self.count() - 1] = index
    self.key_to_index [index]=self.count() - 1
    
  def currentKey(self):
    try:
      return self.index_to_key[self.currentItem()]
    except KeyError:
      return None

  def setCurrentItem(self, key):
    if self.key_to_index.has_key(key):
      QComboBox.setCurrentItem(self, self.key_to_index[key])
  
  def slotItemSelected(self, index):
    rec=self.index_to_key[index]
    self.emit(PYSIGNAL("recSelected"),(rec, index))

  def clear(self):
    QComboBox.clear(self)
    self.index_to_key={}
    self.key_to_index={}

  def initComboBox(self, app, tableName, mode, nullAllowed, currentKey=None):
    self.clear()
    if mode==SEARCH or nullAllowed:
      self.insertItem("<None>", None)
    
    for row in app.getObjects(tableName):
      self.insertItem(row.getDescription(), row.getPrimaryKey())

    if currentKey!=None:
      self.setCurrentItem(currentKey)

  def fillComboBox(self, record, field, mode):
    self.clear()
    if record==None:
      return
    if mode==SEARCH or record.getFieldDefinition(field).nullable==TRUE:
      self.insertItem("<None>", None)
    descriptor=record.getForeignDescriptorColumnName(field)
    fk=record.getForeignKeyColumnName(field)
    for row in record.picklist(field):
      self.insertItem( text=row.getFieldValue(descriptor)
                     , index=row.getFieldValue(fk)
                     )
    self.setCurrentItem(record.getFieldValue(field))

__copyright__="""
/***************************************************************************
    copyright            : (C) 2000 by Boudewijn Rempt 
                           see copyright notice for license
    email                : boud@rempt.xs4all.nl
    Revision             : $Revision: 1.11 $
    Last edited          : $Date: 2001/02/01 19:29:52 $
    
    CVS Log:         
    $Log: guicombobox.py,v $
    Revision 1.11  2001/02/01 19:29:52  boud
    Made more robust.

    Revision 1.10  2001/01/25 23:04:22  boud
    Made non-modal

    Revision 1.9  2001/01/08 20:55:06  boud
    Cleanup for version 1.0

 ***************************************************************************/
"""
