import string

from qt                  import *
from dbobj.dbexceptions  import dbError
from kuragui.constants   import *
from guiconfig           import guiConfig

from kuralib.config      import Config

class guiDetailList(QListView):

  def __init__(self, parent, app, parentRecord, childTable):
    QListView.__init__(self, parent)
    self.setFont(guiConfig.font)
    self.setAllColumnsShowFocus(TRUE)
    self.setFocusPolicy(self.StrongFocus)
    self.setShowSortIndicator(TRUE)

    self.rows={} 

    self.app=app

    self.parentRecord=parentRecord
    self.tableName=childTable
    self.tableDef=self.app.getTableDef(childTable)

    self.headers=[]
    self.setHeaders(self.tableDef)
    
    self.connect(self, SIGNAL("returnPressed(QListViewItem *)")
                     , self.slotItemSelected)
    self.connect(self, SIGNAL("clicked(QListViewItem *)")
                     , self.slotItemSelected)
    self.connect(self, SIGNAL("doubleClicked(QListViewItem *)")
                     , self.slotItemSelected)
    
    self.refresh()
    
  def saveHeaders(self):
    s=[]
    for i in range(len(self.headers)):
      s.append(string.join( ( str(self.columnText(i))
                            , self.headers[i]
                            , str(self.columnWidth(i))
                            ), "."))
    setattr(Config, "lsv_" + self.tableName, string.join(s, ":"))
  
  def setHeaders(self, tableDef):
    # It's a pity, but I can't find a way to find out the exact layout of the columns
    if hasattr(Config, "lsv_" + self.tableName):
      headerdef=getattr(Config, "lsv_" + self.tableName)
    else:
      headerdef=None
    try:
      headers = headerdef.split(":")
      for header in headers:
        label, fieldname, w=header.split(".")
        self.headers.append(fieldname)
        self.addColumn(label, int(w))
    except Exception, err:
      # if something went wrong, delete any column that's been added already
      self.headers=[]
      for i in range(self.columns()):
        self.removeColumn(i)
      # add the cols in standard size
      for fieldname, fieldDef in tableDef.orderedFieldList():
        if fieldDef.inList==TRUE:
          self.headers.append(fieldname)
          self.addColumn(fieldDef.label, 100)

  def refresh(self):
    self.setCursor(Qt.waitCursor)
    QListView.clear(self)
    self.rows={}
    rows=self.parentRecord.getChildren(self.tableName)
  
    for row in rows:
      self.slotAddItem(row)
    self.setCursor(Qt.arrowCursor)
    
  def slotAddItem(self, rec):
    """
      Add a record to the listview and the cache.
    """
    lvi=QListViewItem(self)
    col=0
    for fieldname in self.headers:
      lvi.setText(col, rec.getFieldValue(fieldname))
      col=col+1   
    self.rows[lvi]=rec

  def slotItemSelected(self, lvi):
    if lvi!=None:
      record=self.rows[lvi]
      self.emit(PYSIGNAL("sigRecordSelected"),(record,lvi))

  def slotDeleteItem(self):
    if QMessageBox.warning(self
                  , "Kura"
                  , "Do you want to delete this record?"
                  , "Yes"
                  , "No"
                  ,QString()
                  , 1
                  , 1
                  )==0:
      rec = self.rows[self.currentItem()]
      try:
        rec.delete()
        del self.rows[self.currentItem()]
        self.takeItem(self.currentItem())
      except dbError, dberr:
        QMessageBox.information(self, "Kura Error while deleting", dberr.errorMessage)

      self.emit(PYSIGNAL("sigItemDeleted"),())      

  def slotNewItem(self):
    self.emit(PYSIGNAL("sigCreateNewItem"),(self.parentRecord,))

  def slotUpdateItem(self, rec, lvi):
    col=0
    for fieldname in self.headers:
      lvi.setText(col, rec.getFieldValue(fieldname))
      col=col+1   
    self.rows[lvi]=rec
    
  
  def keyPressEvent(self, e):
    if e.key()==Qt.Key_Enter or e.key()==Qt.Key_Return:
      self.slotItemSelected(self.currentItem())
    elif e.key()==Qt.Key_Delete:
      self.slotDeleteItem()
    elif e.key()==Qt.Key_Insert:
      self.slotNewItem()
    else:
      QListView.keyPressEvent(self, e)


__copyright__="""
/***************************************************************************
    copyright            : (C) 2000 by Boudewijn Rempt 
                           see copyright notice for license
    email                : boud@rempt.xs4all.nl
    Revision             : $Revision: 1.13 $
    Last edited          : $Date: 2001/01/30 20:04:43 $
    
    CVS Log:         
    $Log: guidetaillist.py,v $
    Revision 1.13  2001/01/30 20:04:43  boud
    Removed print statements.

    Revision 1.12  2001/01/30 18:10:25  boud
    Made dialogs non-modal, added remembring of sizes.

    Revision 1.11  2001/01/26 20:01:59  boud
    Move to remembering columns widths.

    Revision 1.10  2001/01/25 23:04:22  boud
    Made non-modal

    Revision 1.9  2001/01/08 20:55:06  boud
    Cleanup for version 1.0

 ***************************************************************************/
"""
