from qt                       import *

from kuragui.guilistbox       import guiListBox
from kuragui.guicombobox      import guiComboBox
from kuragui.guinumberbox     import guiNumberBox
from kuragui.guimultilineedit import guiMultiLineEdit
from kuragui.guicheckbox      import guiCheckBox
from kuragui.guilabel         import guiLabel
from kuragui.guilineedit      import guiLineEdit
from kuragui.constants        import *
from kuragui.guiconfig        import guiConfig

from dbobj.dbtypes            import *

class FieldError(Exception):
  def __init__(self, field):
    self.field=field
  def __repr__(self):
    return "FieldError %s" % self.field

class guiForm(QWidget):
  """
  Generic layout managed form that can show all the fields in a record
  in the right format with the right initial values. This form can be used
  in any layoutmanager.
  """
  def __init__(self, parent, record, mode, *args):
    QWidget.__init__(self, parent)
    self.mode=mode
    self.record=record
    self.grdFields=QGridLayout(self)
    self.grdFields.setSpacing(6)
    self.grdFields.setMargin(11)
    
    self.fieldList=[]
    self.fieldTypes={}
    row=0

    for field, fieldDef in record.tableDef.orderedFieldList():
      if fieldDef.dialog==TRUE: # show in dialog
        self.grdFields.expand(row + 1, 2)
        self.fieldList.append((field, fieldDef))
        self.fieldTypes[field]=fieldDef
        
        if fieldDef.datatype==TEXT: # Multiline
          if fieldDef.readonly and mode != SEARCH:
            fld=self.addField(field, guiLabel(self))
            fld.setAlignment(Qt.AlignLeft or Qt.AlignTop)
            fld.setFrameShape(guiLabel.Panel)
            fld.setFrameShadow(guiLabel.Sunken)
            fld.setFont(guiConfig.font)
          else:
            fld=self.addField(field, guiMultiLineEdit(self)) # create new widget
          fld.setText(record.getFieldValue(field))
          
          self.grdFields.addWidget(self.addField("txt"+field, guiLabel(fld, fieldDef.label, self)), row, 0)
          self.grdFields.addWidget(fld, row, 1)
          
        elif fieldDef.relation <> None: # Combobox
          if fieldDef.readonly and mode != SEARCH:
            fld=self.addField(field, guiLabel(self))
            fld.setFrameShape(guiLabel.Panel)
            fld.setFrameShadow(guiLabel.Sunken)
            fld.setText(record.getFieldValue(record.getDescriptorColumnName(field)))
            fld.setFont(guiConfig.font)
          else:
            fld=self.addField(field, guiComboBox(self))
            fld.fillComboBox(record, field, self.mode)
          
          self.grdFields.addWidget(self.addField("txt"+field, guiLabel(fld, fieldDef.label, self)), row, 0)
          self.grdFields.addWidget(fld, row, 1)
          
        elif fieldDef.datatype==BOOLEAN: # checkbox
          fld=self.addField(field, guiCheckBox(self, fieldDef.label))
          if mode==SEARCH:
            fld.setTristate(TRUE)
          fld.setChecked(record.getFieldValue(field))

          if fieldDef.readonly and mode != SEARCH:
            fld.setEnabled(FALSE)
                      
          self.grdFields.addWidget(fld, row, 0)
          
        elif fieldDef.datatype==INTEGER: # number
          if fieldDef.readonly and mode != SEARCH:
            fld=self.addField(field, guiLabel(self))
            fld.setFrameShape(guiLabel.Panel)
            fld.setFrameShadow(guiLabel.Sunken)
            fld.setFont(guiConfig.font)
          else:
            fld=self.addField(field, guiNumberBox(self))
            fld.setText(record.getFieldValue(field))
          
          self.grdFields.addWidget(self.addField("txt"+field, guiLabel(fld, fieldDef.label, self)), row, 0)
          self.grdFields.addWidget(fld, row, 1)
          
        else:# Lineedit
          if fieldDef.readonly and mode != SEARCH:
            fld=self.addField(field, guiLabel(self))
            fld.setFrameShape(guiLabel.Panel)
            fld.setFrameShadow(guiLabel.Sunken)
            fld.setFont(guiConfig.font)
          else:
            fld=self.addField(field, guiLineEdit(self))
            fld.setMaxLength(fieldDef.length)

          fld.setText(record.getFieldValue(field))
          self.grdFields.addWidget(self.addField("txt"+field, guiLabel(fld, fieldDef.label, self)), row, 0)
          self.grdFields.addWidget(fld, row, 1)
        row=row+1


  def getField(self, fieldname):
    """
      Returns the widget with fieldname
    """
    return getattr(self, fieldname)

  def addField(self, fieldname, widget):
    setattr(self, fieldname, widget)
    return widget

  def setFieldValue(self, field, value):
    if hasattr(self, field):
      fld=self.getField(field)
      if isinstance(fld, guiLineEdit) or isinstance(fld,guiMultiLineEdit):
        fld.setText(unicode(value))
      elif isinstance(fld, guiCheckBox):
        fld.setChecked(value)
      elif isinstance(fld,guiComboBox):
        try: # might be not in combo-list
          fld.setCurrentItem(value)
        except KeyError:
          QMessageBox.warning("Kura internal error"
           , "Value does not exist:" + value + " for field " + field)

  def getFieldText(self, field):
    if hasattr(self, field):
      fld=self.getField(field)
      if (    isinstance(fld, guiLineEdit) 
           or isinstance(fld, guiMultiLineEdit) 
           or isinstance(fld, guiNumberBox)
         ):
        return fld.text()
      elif isinstance(fld,guiCheckBox):
        return fld.isChecked()
      elif isinstance(fld,guiComboBox):
        return fld.currentText()
    else:
      raise FieldError(field)


  def getFieldValue(self, field):
    if hasattr(self, field):
      fld=self.getField(field)
      if (    isinstance(fld, guiLineEdit) 
           or isinstance(fld, guiMultiLineEdit) 
           or isinstance(fld, guiNumberBox)
         ):
        return fld.text()
      elif isinstance(fld,guiCheckBox):
       return fld.isChecked()
      elif isinstance(fld,guiComboBox):
        return fld.currentKey()
    else:
      raise FieldError(field)

  def refreshComboBoxes(self):
    for field, fieldDef in self.record.tableDef.orderedFieldList():
      fld=self.getField(field)
      if isinstance(fld,guiComboBox):
        fld.clear()
        fld.fillComboBox(self.record, field, self.mode)

__copyright__="""
/***************************************************************************
    copyright            : (C) 2000 by Boudewijn Rempt 
                           see copyright notice for license
    email                : boud@rempt.xs4all.nl
    Revision             : $Revision: 1.17 $
    Last edited          : $Date: 2001/02/01 19:29:52 $
    
    CVS Log:         
    $Log: guiform.py,v $
    Revision 1.17  2001/02/01 19:29:52  boud
    Made more robust.

    Revision 1.16  2001/01/30 20:04:44  boud
    Removed print statements.

    Revision 1.15  2001/01/08 20:55:07  boud
    Cleanup for version 1.0

 ***************************************************************************/
"""
