import sys, string
from types import NoneType, StringType, IntType

from qt import *

from dbobj.dbexceptions  import dbError
from guitabdialog import guiTabDialog
from guiconfig import guiConfig
from constants import *

from kuralib.config import Config

checkmark = [
"10 12 3 1",
" 	c None",
".	c #000000",
"+	c #A2A2A2",
"       ...",
"      .++.",
"      .+. ",
"     .++. ",
".    .+.  ",
"..  .++.  ",
".+. .+.   ",
".++.++.   ",
" .+++.    ",
"  .++.    ",
"   ..     ",
"    .     "
]

class guiListView(QListView):
  """
    Updated & improved listview component
    
    Show a set of records in a listview.
    
    Supports:
            function   standard binding           Calls
    insert: New()      Insert                     Master-Detail form
    update: Open()     Enter/Return/doubleclick   Master-Detail form
    delete: Delete()   Delete                     messagebox for confirmation
    query : Find()     F2                         Master-only form
             
  """

  def __init__( self, app, tableName
              , searchRec, dlgForm, dlgSearch
              , parent, showChildren=FALSE, *args):
    apply(QListView.__init__,(self, parent, ) + args)
    self.checkmark=QPixmap(checkmark)
    self.nocheck=QPixmap([""])
    self.setFont(guiConfig.font)
    self.setAllColumnsShowFocus(TRUE)
    self.setFocusPolicy(self.StrongFocus)
    self.setShowSortIndicator(TRUE)
    self.showChildren=showChildren
    self.rows={} 
    self.app=app
    self.tableName=tableName
    self.masterLabel=self.app.getTableLable(tableName)
    self.tableDef=self.app.getTableDef(tableName)
    self.searchRec=searchRec
    
    if dlgForm==None:
      self.dlgFormClass=guiTabDialog
    else:
      self.dlgFormClass=dlgForm
    if dlgSearch==None:
      self.dlgSearchClass=guiTabDialog
    else:
      self.dlgSearchClass=dlgSearch
    
    self.headers=[]
    self.setHeaders(self.tableDef)
    
    if self.searchRec<>None:
      self.refresh(self.searchRec)

  def saveHeaders(self):
    s=[]
    for i in range(len(self.headers)):
      s.append(string.join( ( str(self.columnText(i))
                            , self.headers[i]
                            , str(self.columnWidth(i))
                            ), "."))
    setattr(Config, "lsv_" + self.tableName, string.join(s, ":"))
  
  def setHeaders(self, tableDef):
    # It's a pity, but I can't find a way to find out the exact layout of the columns
    if hasattr(Config, "lsv_" + self.tableName):
      headerdef=getattr(Config, "lsv_" + self.tableName)
    else:
      headerdef=None
    try:
      headers = headerdef.split(":")
      for header in headers:
        label, fieldname, w=header.split(".")
        self.headers.append(fieldname)
        self.addColumn(label, int(w))
    except Exception, err:
      # if something went wrong, delete any column that's been added already
      self.headers=[]
      for i in range(self.columns()):
        self.removeColumn(i)
      # add the cols in standard size
      for fieldname, fieldDef in tableDef.orderedFieldList():
        if fieldDef.inList==TRUE:
          self.headers.append(fieldname)
          self.addColumn(fieldDef.label, 100)


  def addItem(self, rec):
    """
      Add a record to the listview and the cache.
    """
    lvi=QListViewItem(self)
    col=0
    for fieldname in self.headers:
      value=rec.getFieldValue(fieldname)
      if rec.getFieldDefinition(fieldname).datatype==BOOLEAN:
        if rec.getFieldValue(fieldname):
          lvi.setPixmap(col, self.checkmark)
      else:    
        if type(value)==IntType:
          value=str(value)
        lvi.setText(col, value)
      col=col+1   
    self.rows[lvi]=rec
    return lvi
  
  def refresh(self, searchRec):
    """
      Reload the listview
    """
    self.setCursor(Qt.waitCursor)
    QListView.clear(self)
    self.rows={}
    rows=self.app.getObjects(self.tableName, fields=searchRec.getFields())
    for row in rows:
      self.addItem(row)
    self.setCursor(Qt.arrowCursor)
    
  def Find(self):
    if Config.useDefaultForSearch==TRUE:
      record=self.app.createDefaultObject(self.tableName)
    else:
      record=self.app.createObject(self.tableName)
      
    self.dlgSearch=self.dlgSearchClass(self.app, self, "Find " + self.masterLabel, self.masterLabel
        , record, SEARCH, self.tableDef, showChildren=FALSE)

    self.connect(self.dlgSearch, PYSIGNAL("sigAcceptData")
                               , self.slotFindAccept)
    self.dlgSearch.show()
    
  def slotFindAccept(self):
    self.refresh( self.dlgSearch.getMasterRecord() )
    
  def New(self):
    defaultRec=self.app.createDefaultObject(self.tableName)

    self.dlgNew=self.dlgFormClass(app=self.app , parent=self, title="New " + self.masterLabel
      , firstTabTitle=self.masterLabel, record=defaultRec, mode=INSERT
      , tableDef=self.tableDef, showChildren=FALSE
      )

    self.connect(self.dlgNew, PYSIGNAL("sigAcceptData")
                               , self.slotNewAccept)
    self.dlgNew.show()

  def slotNewAccept(self):
    record=self.dlgNew.getMasterRecord()
    lvi=self.addItem(record)
    self.ensureItemVisible(lvi)
         
  def Open(self):
    try:
      record=self.rows[self.currentItem()]
    except KeyError: 
      # Probably an empty list
      return 
    
    self.dlgOpen=self.dlgFormClass(self.app, self, 'Edit ' + self.masterLabel
                   , self.masterLabel,  record,  UPDATE, self.tableDef,
                   self.showChildren)
                   
    self.connect(self.dlgOpen, PYSIGNAL("sigAcceptData")
                             , self.slotOpenAccept)
    
    self.dlgOpen.show()
    
  def slotOpenAccept(self): 
    rec=self.dlgOpen.getMasterRecord()
    self.rows[self.currentItem()]=rec
    col=0
    for (fieldname, field) in self.tableDef.orderedFieldList():
      if field.inList==TRUE:
        if field.datatype==BOOLEAN:
          if rec.getFieldValue(fieldname):
            self.currentItem().setPixmap(col, self.checkmark)
          else:
            self.currentItem().setPixmap(col, self.nocheck)
        else:
          self.currentItem().setText(col, rec.getFieldValue(fieldname))
        col=col+1

  def Delete(self):
    
    if QMessageBox.warning(self
                  , "Kura"
                  , "Do you want to delete this record?"
                  , "Yes"
                  , "No"
                  ,QString()
                  , 1
                  , 1
                  )==0:
      rec = self.rows[self.currentItem()]
      try:
        rec.delete()
        del self.rows[self.currentItem()]
        self.takeItem(self.currentItem())
      except dbError, dberr:
        QMessageBox.information(self, "Kura Error while deleting", dberr.errorMessage)


  def contentsMouseDoubleClickEvent(self, e):
    self.Open()

  def keyPressEvent(self, e):
    if e.key()==Qt.Key_Enter or e.key()==Qt.Key_Return:
      self.Open()
    elif e.key()==Qt.Key_F2:
      self.Find()
    elif e.key()==Qt.Key_Delete:
      self.Delete()
    elif e.key()==Qt.Key_Insert:
      self.New()
    else:
      QListView.keyPressEvent(self, e)

def main():

  try:
    from kuralib import lngapp
  except:
    sys.path.append("..")
    from kuralib import lngapp

  from kuralib.lng_lex import *

  qapp=QApplication(sys.argv)
  app=lngapp.lngApp(host="localhost", user="boud", database="lng")
  toplevel=guiListView( app
                      , tableName="lng_lex"
                      , searchRec=app.createObject('lng_lex', form='adan')
                      , dlgForm=None
                      , dlgSearch=None
                      , parent=None
                      )
  toplevel.show()
  qapp.connect(qapp, SIGNAL('lastWindowClosed()'), qapp, SLOT('quit()'))
  print qapp.exec_loop()
  
if __name__=="__main__":
  if len(sys.argv)>1:
    if sys.argv[1]=='-p':
      import profile
      profile.run('main()')
  else:
    main()

__copyright__="""
/***************************************************************************
    copyright            : (C) 2000 by Boudewijn Rempt 
                           see copyright notice for license
    email                : boud@rempt.xs4all.nl
    Revision             : $Revision: 1.20 $
    Last edited          : $Date: 2001/01/30 20:04:44 $
    
    CVS Log:         
    $Log: guilistview.py,v $
    Revision 1.20  2001/01/30 20:04:44  boud
    Removed print statements.

    Revision 1.19  2001/01/30 19:39:17  boud
    Updated  elements are once again updated in the listview.

    Revision 1.18  2001/01/30 18:10:25  boud
    Made dialogs non-modal, added remembring of sizes.

    Revision 1.17  2001/01/26 20:01:59  boud
    Move to remembering columns widths.

    Revision 1.16  2001/01/25 23:04:22  boud
    Made non-modal

    Revision 1.15  2001/01/08 20:55:07  boud
    Cleanup for version 1.0

 ***************************************************************************/
"""
