from dbobj.dbobj import dbRecord, dbTable
from dbobj.dbexceptions import *
from dbobj.dbtypes import *
from string import rstrip, lstrip, join

from lng_lex import lng_lex, lng_lexemes
from lng_eltg import lng_element_tag

from config import Config

class lng_element(dbRecord):

  def __init__(self, app, **args):
    if args.has_key("fields"):
      dbRecord.__init__(self, app, "lng_element", args["fields"])
    else:
      dbRecord.__init__(self, app, "lng_element", args)
    self.elements=[]
    self.tags=[]

  def getTags(self):
    self.tags=self.getChildren("lng_element_tag")
    return self.tags

  def getElements(self):
    self.elements=self.getChildren("lng_element")
    return self.elements

  def buildElementTree(self):
    self.getTags()
    for element in self.getElements():
      element.buildElementTree()
   
  def translation(self):
    if (self.tags==[] or self.tags==None):
      self.getTags()
    for tag in self.tags:
      if tag.tag=="TR" or tag.tag=="GL" or tag.tag=="GLOSS":
        return tag.getDescription()
    
    if self.lexeme <> "" and self.lexeme <> None:
      return self.lexeme
    else:
      try:
        lexeme  = self.app.getObject("lng_lex", form=self.text, languagenr=self.languagenr)
        return lexemes[0].glosse
      except:
        return "#"


  def getGlosse(self):
    try:
      glosse=Config.app.getObject("lng_element_tag", elementnr=self.elementnr
                                                   , tag="GLOSS"
                                                   )
      return glosse.value
    except dbRecordNotFoundException, error:
      print "No glosse"
    except dbTooManyRowsException, error:
      print "Too many glosses"
    
  
  def getPhoneticTranscription(self):
    try:
      phonetic_transcription=Config.app.getObject("lng_element_tag", elementnr=self.elementnr
                                                                   , tag="PHON"
                                                                   )
      return phonetic_transcription.value
    except dbRecordNotFoundException, error:
      print "No glosse"
    except dbTooManyRowsException, error:
      print "Too many glosses"
    return None

  def getDescription(self):
    try:
      phonetic_transcription=Config.app.getObject("lng_element_tag", elementnr=self.elementnr
                                                                   , tag="NOTE"
                                                                   )
      return phonetic_transcription.value
    except dbRecordNotFoundException, error:
      print "No glosse"
    except dbTooManyRowsException, error:
      print "Too many glosses"
    return None
          
  def getTag(self, tag):
    try:
      tagRecord=self.app.getObject("lng_element_tag", elementnr=self.elementnr, tag=tag)
      return tagRecord
    except dbRecordNotFoundException, error:
      tagRecord=error.queryRec
      return tagRecord
    return None
    
  def elmtLength(self):
    return max( len(self.text), len(self.translation()) )

  def elmtHTML(self):
    if self.lexnr<> 0 and self.lexnr <> None:
      e=join(( '<td align="top"><a href="/KURA?command=record?table=lng_element?elementnr='
                  , str(self.elementnr)
                  , '">'
                  , self.text
                  , '</a><br><a href="/KURA?command=record?table=lng_lex?lexnr='
                  , str(self.lexnr)
                  , '">'
                  , self.translation()
                  , '</a></td>'
                 ),"")
    else:
      e=join(( '<td align="top"><a href="/KURA?command=record?table=lng_element?elementnr='
                  , str(self.elementnr)
                  , '">'
                  , self.text
                  , '</a><br><a href="/KURA?command=record?table=lng_lex?form='
                  , self.text
                  , '?languagenr='
                  , str(self.languagenr)
                  , '">'
                  , self.translation()
                  , '</a></td>'
                 ),"")
    return (e, self.elmtLength())

  def type(self):
    type=Config.app.getObject("lng_elementtypecode", elementtypecode=self.elementtypecode)
    return type
    
class lng_elements(dbTable):
  """
    Collection class for all element data.
  """
  def __init__(self, app):
    dbTable.__init__(self, app, table="lng_element", recObj=lng_element)

  def insert(self, streamnr, languagenr=None, elementTexts=[]):
    seqnr=0
    if languagenr==None:
      languagenr=Config.languagenr
      
    for elementText in elementTexts:
      if elementText <> "":
        element=lng_element( self.app
                           , streamnr=streamnr
                           , seqnr=seqnr
                           , text=rstrip(lstrip(elementText))
                           , usernr=Config.usernr
                           , languagenr=languagenr
                           )
        element.insert(checkIntegrity=FALSE)
        seqnr=seqnr + 1

  def recordHTML(self, out, keys={}, includeChildTables=FALSE):
    dbTable.recordHTML(self, out, keys, TRUE)
__copyright__="""
/***************************************************************************
    copyright            : (C) 2000 by Boudewijn Rempt 
                           see copyright notice for license
    email                : boud@rempt.xs4all.nl
    Revision             : $Revision: 1.9 $
    Last edited          : $Date: 2001/01/16 20:58:41 $
    
    CVS Log:         
    $Log: lng_elmt.py,v $
    Revision 1.9  2001/01/16 20:58:41  boud
    ...

    Revision 1.8  2001/01/15 20:21:15  boud
    Added import/export filters, bugfixes.

    Revision 1.7  2001/01/08 20:55:07  boud
    Cleanup for version 1.0

 ***************************************************************************/
"""
