import xmllib, os, os.path
import kurahtmlwriter
from dbobj.dbobj import dbTable

#
# Install the htmlwriter mix-in. (Thanks to Chuck Esterbrook for the idea!)
#
kurahtmlwriter.installWriter(dbTable)

TRUE=1
FALSE=0

class KuraParser(xmllib.XMLParser): 

  def __init__(self, app):
    xmllib.XMLParser.__init__(self)
    self.app=app
    self.docvars={}
    self.links=[]
    self.templates={'link'  : open(os.path.join(os.environ["KURASERVERDIR"],'templates/tpl_link.html'),'r').read()
                   ,'header': open(os.path.join(os.environ["KURASERVERDIR"],'templates/tpl_header.html'),'r').read()
                   ,'footer': open(os.path.join(os.environ["KURASERVERDIR"],'templates/tpl_footer.html'),'r').read()
                   }

    self.elements={'kura_document': (self.start_kura_document, self.end_kura_document)
                  ,'kura_record'  : (self.start_kura_record, self.end_kura_record)
                  ,'kura_contents': (self.start_kura_contents, self.end_kura_contents)
                  ,'kura_list'    : (self.start_kura_list, self.end_kura_list)
                  ,'kura_link'    : (self.start_kura_link, self.end_kura_link)
                  ,'kura_footer'  : (self.start_kura_footer, self.end_kura_footer)
                  }

  def doReset(self, out):
    xmllib.XMLParser.reset(self)
    self.out=out
    self.docvars={}
    self.links=[]
    
  def start_kura_document(self, attrs):
    self.docvars=attrs
    self.out(self.templates['header'] % self.docvars)
    
  def end_kura_document(self):
    self.out('</html>')

  def start_kura_contents(self, attrs):
    if attrs.has_key("table") and attrs.has_key("indexfield"):
      table=attrs["table"]    
      indexfield=attrs["indexfield"]
      del attrs["indexfield"]
      del attrs["table"]
      tblObj=self.app.createTableObject(table)
      tblObj.contentsHTML(self.out, indexfield, attrs)
    else:
      self.out("<p>Couldn't perform kura_contents command.</p>")
    
  def end_kura_contents(self):
    pass

  def start_kura_link(self, attrs):
    print attrs, self.templates['link']
    self.links.append(attrs) 
    self.out(self.templates['link'] % attrs)

  def end_kura_link(self):
    pass
 
  def start_kura_footer(self, attrs):
    self.out(self.templates['footer'] % self.docvars )

  def end_kura_footer(self):
    pass

  def start_kura_list(self, attrs):
    if attrs.has_key("table"):
      table=attrs["table"]
      del attrs["table"]
      print table
      tblObj=self.app.createTableObject(table)
      
      tblObj.listHTML(self.out, attrs)
    else:
      self.out("<p>Couldn't perform kuralist command.</p>")

  def end_kura_list(self): pass
  
  def start_kura_record(self, attrs):
    if attrs.has_key("table"):
      table=attrs["table"]
      del attrs["table"]
      tblObj=self.app.createTableObject(table)
      if attrs.has_key("includechildtables"):
        tblObj.recordHTML(self.out, attrs, TRUE)
      else:
        tblObj.recordHTML(self.out, attrs)
    else:
      self.out("<p>Couldn't perform kurarecord command.</p>")
 
  def end_kura_record(self): pass
  
  def unknown_starttag(self, gi, attrs):
    self.out("<" + gi + " ")
    for k, v in attrs.items():
      self.out(k + '="' + v + '"')
    self.out(">")
  
  def unknown_endtag(self, gi):
    self.out(r"</" + gi + ">" )
    
  def handle_data(self, data):
    self.out(data)
    
  def handle_cdata(self, data):
    self.out(data)

class KuraDocBuilder:
  
  def __init__(self, app):
    self.app=app
    self.parser=KuraParser(self.app)
 
  def document(self, document, wfile):
    self.parser.doReset(wfile.write)
    self.parser.feed(document)
    self.parser.close()


__copyright__="""
/***************************************************************************
    copyright            : (C) 2000 by Boudewijn Rempt 
                           see copyright notice for license
    email                : boud@rempt.xs4all.nl
    Revision             : $Revision: 1.2 $
    Last edited          : $Date: 2001/01/08 20:55:07 $
    
    CVS Log:         
    $Log: kuradocbuilder.py,v $
    Revision 1.2  2001/01/08 20:55:07  boud
    Cleanup for version 1.0

 ***************************************************************************/
"""
