#!/usr/bin/env python
"""
  Custom webserver:
  
  converts xml documents with embedded kura commands to html
  carries out kura commands (KURA?xx=yy?...)
  serves up plain html, gif, png and realaudio
  
"""
import sys
sys.setappdefaultencoding("utf-8")
import os
import time
import socket
import string
import SocketServer
import os.path
import BaseHTTPServer
import SimpleHTTPServer

# Configuration - must be loaded before lngapp
from kuralib.config import Config

# Kura application object
from kuralib import lngapp

# Command controller
from kuracontroller import KuraController

# xml2html doc builder
from kuradocbuilder import KuraDocBuilder


class HTTPServer(SocketServer.ThreadingTCPServer):
  """
    Multithreaded server
  """
  def server_bind(self):
    """Override server_bind to store the server name."""
    SocketServer.ThreadingTCPServer.server_bind(self)
    host, port = self.socket.getsockname()
    if not host or host == '0.0.0.0':
      host = socket.gethostname()
    hostname, hostnames, hostaddrs = socket.gethostbyaddr(host)
    if '.' not in hostname:
      for host in hostnames:
        if '.' in host:
          hostname = host
          break
    self.server_name = hostname
    self.server_port = port


class KuraHTTPServer(HTTPServer):
  """
    Store some important initialisation objects
  """
  def __init__(self, serveraddress, RequestHandlerClass):
    HTTPServer.__init__(self
           , serveraddress
           , RequestHandlerClass
           )
    self.app=lngapp.lngApp(user=Config.dbuser
                        ,database=Config.database
                        ,host=Config.host
                        ,password=Config.password
                        )
    self.ctrl=KuraController(self.app)
    self.docbuilder=KuraDocBuilder(self.app)

class KuraHTTPRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler): 
 
  server_version="KuraServer/"+"0.2"
  extensions_map = {
            '': 'text/plain',   # Default, *must* be present
            '.html': 'text/html; charset=ISO-10646-1',
            '.htm': 'text/html; charset=ISO-10646-1',
            '.gif': 'image/gif',
            '.png': 'image/png',
            '.xpm': 'image/xpm',
            '.jpg': 'image/jpeg',
            '.jpeg': 'image/jpeg',
            '.rm': 'audio/x-pn-realaudio',
            '.ra': 'audio/x-pn-realaudio',
            '.wav': 'audio/wav',
            '.mp3': 'audio/mpeg3',
            '.xml': 'text/xml; charset=ISO-10646-1',
            }
            
  error_message_format="""\
<head>
<title>Kura Error Response</title>
</head>
<body>
<h1>Error response</h1>
<p>Error code %(code)d.
<p>Message: %(message)s.
<p>Error code explanation: %(code)s = %(explain)s.
</body>
"""
  documentroot=os.path.join(os.environ["KURASERVERDIR"],"documents")
  logdir=os.environ["KURASERVERDIR"]
  

  def log_message(self, format, *args):
    self.logfile=open(os.path.join(self.logdir,"kura.log"),"a")
    self.logfile.write("%s - - [%s] %s\n" %
                         ( self.address_string()
                         , self.log_date_time_string()
                         , format%args
                         )
                      )
    self.logfile.flush()
 
  def sendHead(self, path):
    f=None
    content_type=None
    if path[1:5]=="KURA":
      content_type="text/html"
      f="KURA"
      path=path[6:]
    else:
      path=os.path.join(self.documentroot, path[1:])
      if os.path.isdir(path):
        self.send_error(403, "Directory listing not supported")
        return None, path
      else:
        try:
          f = open(path, 'rb')
        except IOError:
          self.send_error(404, "File not found")
          return None, path
    self.send_response(200)
    self.send_header("Content-type", content_type or self.guess_type(path))
    self.end_headers()
    return f, path
   
  def do_GET(self):

    if self.path=="/":     # Expand the root document
      f, path=self.sendHead("/root.html")
    else:
      f, path=self.sendHead(self.path)  
    
    # determine request subtype: command, xml file or plain file transfer 
    if f=="KURA":
      r=self.server.ctrl.execute(path, self.wfile)
    elif f:
      base, ext = os.path.splitext(path)
      # XML pages must be converted to html
      if ext == '.XML' or ext == '.xml': 
        # interpreted XML files
        r=self.server.docbuilder.document(f.read(), self.wfile)
        f.close()
      else:
        # in all other cases, just copy the files
        self.copyfile(f, self.wfile)
        f.close()


def start( HandlerClass=KuraHTTPRequestHandler
         , ServerClass = KuraHTTPServer):
  """Test the HTTP request handler class.
l
  This runs an HTTP server on port 8000 (or the first command line
  argument).

  """

  if sys.argv[1:]:
      port = string.atoi(sys.argv[1])
  else:
      port = 8000
  server_address = ('', port)

  httpd = ServerClass(server_address, HandlerClass)

  print "Starting the Kura http server on port", port, "..."
  httpd.serve_forever()

if __name__=="__main__":
  start()

__copyright__="""
/***************************************************************************
    copyright            : (C) 2000 by Boudewijn Rempt 
                           see copyright notice for license
    email                : boud@rempt.xs4all.nl
    Revision             : $Revision: 1.6 $
    Last edited          : $Date: 2001/01/30 20:04:46 $
    
    CVS Log:         
    $Log: kurasrvr.py,v $
    Revision 1.6  2001/01/30 20:04:46  boud
    Removed print statements.

    Revision 1.5  2001/01/08 20:55:07  boud
    Cleanup for version 1.0

 ***************************************************************************/
"""
